@@ -9,13 +9,10 @@ from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 from trytond.tools import grouped_slice
 
-__all__ = ['PurchaseRequest', 'PurchaseConfiguration',
-    'PurchaseConfigurationSupplyPeriod']
 supply_period = fields.TimeDelta("Supply Period")
 
 
-class PurchaseConfiguration:
-    __metaclass__ = PoolMeta
+class PurchaseConfiguration(metaclass=PoolMeta):
     __name__ = 'purchase.configuration'
     supply_period = fields.MultiValue(supply_period)
 
@@ -26,22 +23,9 @@ class PurchaseConfigurationSupplyPeriod(ModelSQL, ValueMixin):
     supply_period = supply_period
 
 
-class PurchaseRequest:
+class PurchaseRequest(metaclass=PoolMeta):
     'Purchase Request'
     __name__ = 'purchase.request'
-    __metaclass__ = PoolMeta
-
-    @classmethod
-    def __register__(cls, module_name):
-        cursor = Transaction().connection.cursor()
-        sql_table = cls.__table__()
-        super(PurchaseRequest, cls).__register__(module_name)
-
-        # Migration from 2.0: empty order point origin is -1 instead of 0
-        cursor.execute(*sql_table.update(
-                columns=[sql_table.origin],
-                values=['stock.order_point,-1'],
-                where=sql_table.origin == 'stock.order_point,0'))
 
     @classmethod
     def _get_origin(cls):
@@ -102,12 +86,13 @@ class PurchaseRequest:
         # aggregate product by minimum supply date
         date2products = {}
         for product in products:
-            min_date, max_date = cls.get_supply_dates(product)
+            min_date, max_date = cls.get_supply_dates(
+                product, company=company.id)
             date2products.setdefault((min_date, max_date), []).append(product)
 
         # compute requests
         new_requests = []
-        for dates, dates_products in date2products.iteritems():
+        for dates, dates_products in date2products.items():
             min_date, max_date = dates
             for sub_products in grouped_slice(dates_products):
                 sub_products = list(sub_products)
@@ -115,9 +100,9 @@ class PurchaseRequest:
                 with Transaction().set_context(forecast=True,
                         stock_date_end=min_date or datetime.date.max):
                     pbl = Product.products_by_location(warehouse_ids,
-                        product_ids, with_childs=True)
+                        with_childs=True, grouping_filter=(product_ids,))
                 for warehouse_id in warehouse_ids:
-                    min_date_qties = defaultdict(lambda: 0,
+                    min_date_qties = defaultdict(int,
                         ((x, pbl.pop((warehouse_id, x), 0))
                             for x in product_ids))
                     # Do not compute shortage for product
@@ -150,7 +135,8 @@ class PurchaseRequest:
                 ('purchase_line', '=', None),
                 ('origin', 'like', 'stock.order_point,%'),
                 ])
-        reqs = [r for r in reqs if r.product in products]
+        reqs = [r for r in reqs
+            if r.product in products and r.warehouse in warehouses]
         cls.delete(reqs)
         new_requests = cls.compare_requests(new_requests)
 
@@ -178,7 +164,7 @@ class PurchaseRequest:
 
         requests = Request.search([
                 ('purchase_line.moves', '=', None),
-                ('purchase_line.purchase.state', '!=', 'cancel'),
+                ('purchase_line.purchase.state', '!=', 'cancelled'),
                 ('origin', 'like', 'stock.order_point,%'),
                 ])
         # Fetch data from existing requests
@@ -202,7 +188,7 @@ class PurchaseRequest:
                         'quantity': quantity,
                         })
 
-        for i in existing_req.itervalues():
+        for i in existing_req.values():
             i.sort(key=lambda r: r['supply_date'])
 
         # Update new requests to take existing requests into account
@@ -215,7 +201,8 @@ class PurchaseRequest:
                         new_req.computed_quantity - old_req['quantity'])
                     new_req.quantity = Uom.compute_qty(
                         new_req.product.default_uom, new_req.computed_quantity,
-                        new_req.uom)
+                        new_req.uom, round=False)
+                    new_req.quantity = new_req.uom.ceil(new_req.quantity)
                     old_req['quantity'] = max(0.0,
                         old_req['quantity'] - new_req.computed_quantity)
                 else:
@@ -224,7 +211,7 @@ class PurchaseRequest:
         return new_requests
 
     @classmethod
-    def get_supply_dates(cls, product):
+    def get_supply_dates(cls, product, **pattern):
         """
         Return the minimal interval of earliest supply dates for a product.
         """
@@ -234,7 +221,7 @@ class PurchaseRequest:
         max_date = None
         today = Date.today()
 
-        for product_supplier in product.product_suppliers:
+        for product_supplier in product.product_suppliers_used(**pattern):
             supply_date = product_supplier.compute_supply_date(date=today)
             next_day = today + product_supplier.get_supply_period()
             next_supply_date = product_supplier.compute_supply_date(
@@ -256,7 +243,8 @@ class PurchaseRequest:
 
     @classmethod
     def compute_request(cls, product, location_id, shortage_date,
-            product_quantity, company, order_point=None):
+            product_quantity, company, order_point=None,
+            supplier_pattern=None):
         """
         Return the value of the purchase request which will answer to
         the needed quantity at the given date. I.e: the latest
@@ -267,15 +255,22 @@ class PurchaseRequest:
         Uom = pool.get('product.uom')
         Request = pool.get('purchase.request')
 
+        if supplier_pattern is None:
+            supplier_pattern = {}
+        else:
+            supplier_pattern = supplier_pattern.copy()
+        supplier_pattern['company'] = company.id
+
         supplier, purchase_date = cls.find_best_supplier(product,
-            shortage_date)
+            shortage_date, **supplier_pattern)
 
         uom = product.purchase_uom or product.default_uom
         target_quantity = order_point.target_quantity if order_point else 0.0
         computed_quantity = target_quantity - product_quantity
-        product_quantity = uom.round(product_quantity)
+        product_quantity = uom.ceil(product_quantity)
         quantity = Uom.compute_qty(
-            product.default_uom, computed_quantity, uom)
+            product.default_uom, computed_quantity, uom, round=False)
+        quantity = uom.ceil(quantity)
 
         if order_point:
             origin = 'stock.order_point,%s' % order_point.id
@@ -345,8 +340,8 @@ class PurchaseRequest:
                     stock_date_start=current_date,
                     stock_date_end=current_date):
                 pbl = Product.products_by_location([location_id],
-                    product_ids, with_childs=True)
-            for key, qty in pbl.iteritems():
+                    with_childs=True, grouping_filter=(product_ids,))
+            for key, qty in pbl.items():
                 _, product_id = key
                 current_qties[product_id] += qty
 
