@@ -2,41 +2,14 @@
 # this repository contains the full copyright notices and license terms.
 import datetime
 
-from sql import Table
-from sql.functions import Overlay, Position
-
 from trytond.model import ModelView, ModelSQL
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 
-__all__ = ['ShipmentInternal']
-
 
 class ShipmentInternal(ModelSQL, ModelView):
     __name__ = 'stock.shipment.internal'
 
-    @classmethod
-    def __register__(cls, module_name):
-        cursor = Transaction().connection.cursor()
-        model_data = Table('ir_model_data')
-        model = Table('ir_model')
-        # Migration from 1.2: packing renamed into shipment
-        cursor.execute(*model_data.update(
-                columns=[model_data.fs_id],
-                values=[Overlay(model_data.fs_id, 'shipment',
-                        Position('packing', model_data.fs_id),
-                        len('packing'))],
-                where=model_data.fs_id.like('%packing%')
-                & (model_data.module == module_name)))
-        cursor.execute(*model.update(
-                columns=[model.model],
-                values=[Overlay(model.model, 'shipment',
-                        Position('packing', model.model),
-                        len('packing'))],
-                where=model.model.like('%packing%')
-                & (model.module == module_name)))
-        super(ShipmentInternal, cls).__register__(module_name)
-
     @classmethod
     def generate_internal_shipment(cls, clean=True):
         """
@@ -82,7 +55,7 @@ class ShipmentInternal(ModelSQL, ModelView):
                 ('overflowing_location', '!=', None),
                 ])
         id2location.update({l.id: l for l in implicit_locations})
-        location_ids = id2location.keys()
+        location_ids = list(id2location.keys())
 
         # ordered by ids to speedup reduce_ids in products_by_location
         if implicit_locations:
@@ -91,12 +64,12 @@ class ShipmentInternal(ModelSQL, ModelView):
                     ], order=[('id', 'ASC')])
             product_ids = [p.id for p in products]
         else:
-            product_ids = id2product.keys()
+            product_ids = list(id2product.keys())
             product_ids.sort()
 
         with Transaction().set_context(forecast=True, stock_date_end=today):
             pbl = Product.products_by_location(
-                location_ids, product_ids, with_childs=True)
+                location_ids, with_childs=True, grouping_filter=(product_ids,))
 
         shipments = []
         date = today
@@ -105,7 +78,7 @@ class ShipmentInternal(ModelSQL, ModelView):
         while date <= end_date:
             # Create a list of moves to create
             moves = {}
-            for location in id2location.itervalues():
+            for location in id2location.values():
                 for product_id in product_ids:
                     qty = current_qties.get((location.id, product_id), 0)
                     op = product2op.get((location.id, product_id))
@@ -143,12 +116,12 @@ class ShipmentInternal(ModelSQL, ModelView):
 
             # Group moves by {from,to}_location
             to_create = {}
-            for key, qty in moves.iteritems():
+            for key, qty in moves.items():
                 from_location, to_location, product = key
                 to_create.setdefault(
                     (from_location, to_location), []).append((product, qty))
             # Create shipments and moves
-            for locations, moves in to_create.iteritems():
+            for locations, moves in to_create.items():
                 from_location, to_location = locations
                 shipment = cls(
                     from_location=from_location,
@@ -182,8 +155,10 @@ class ShipmentInternal(ModelSQL, ModelView):
                     stock_date_start=date,
                     stock_date_end=date):
                 pbl = Product.products_by_location(
-                    location_ids, product_ids, with_childs=True)
-            for key, qty in pbl.iteritems():
+                    location_ids,
+                    with_childs=True,
+                    grouping_filter=(product_ids,))
+            for key, qty in pbl.items():
                 current_qties[key] += qty
 
         if shipments:
