@@ -10,7 +10,8 @@ from trytond.tests.test_tryton import doctest_teardown
 from trytond.tests.test_tryton import doctest_checker
 from trytond.pool import Pool
 
-from trytond.modules.company.tests import create_company, set_company
+from trytond.modules.company.tests import (
+    create_company, set_company, CompanyTestMixin)
 from trytond.modules.account.tests import create_chart
 
 DATES = [
@@ -32,7 +33,7 @@ DATES = [
     ]
 
 
-class StockSupplyTestCase(ModuleTestCase):
+class StockSupplyTestCase(CompanyTestMixin, ModuleTestCase):
     'Test StockSupply module'
     module = 'stock_supply'
 
@@ -98,11 +99,11 @@ class StockSupplyTestCase(ModuleTestCase):
         with set_company(company):
             create_chart(company)
             receivable, = Account.search([
-                ('kind', '=', 'receivable'),
+                ('type.receivable', '=', True),
                 ('company', '=', company.id),
                 ])
             payable, = Account.search([
-                ('kind', '=', 'payable'),
+                ('type.payable', '=', True),
                 ('company', '=', company.id),
                 ])
             supplier, = Party.create([{
@@ -111,13 +112,56 @@ class StockSupplyTestCase(ModuleTestCase):
                         'account_payable': payable.id,
                         }])
             product_supplier, = ProductSupplier.create([{
-                        'product': template.id,
+                        'template': template.id,
                         'company': company.id,
                         'party': supplier.id,
                         'lead_time': lead_time,
                         }])
             return product_supplier
 
+    @with_transaction()
+    def test_order_point_location_searcher(self):
+        pool = Pool()
+        Uom = pool.get('product.uom')
+        Template = pool.get('product.template')
+        Product = pool.get('product.product')
+        Location = pool.get('stock.location')
+        OrderPoint = pool.get('stock.order_point')
+        unit, = Uom.search([('symbol', '=', 'u')])
+        template, = Template.create([{
+                    'name': 'ProductTest',
+                    'type': 'goods',
+                    'default_uom': unit.id,
+                    'purchase_uom': unit.id,
+                    'list_price': Decimal(0),
+                    'purchasable': True,
+                    }])
+        product, = Product.create([{
+                    'template': template.id,
+                    }])
+
+        warehouse, = Location.search([('type', '=', 'warehouse')])
+        storage, = Location.search([('code', '=', 'STO')])
+
+        company = create_company()
+        with set_company(company):
+            order_point, = OrderPoint.create([{
+                        'product': product.id,
+                        'type': 'purchase',
+                        'warehouse_location': warehouse.id,
+                        'min_quantity': 0.0,
+                        'target_quantity': 5.0,
+                        }])
+
+            for clause, result in [
+                    (('location', '=', warehouse.name), [order_point]),
+                    (('location', '=', 'storage'), []),
+                    (('location', '!=', warehouse.name), []),
+                    (('location', '!=', 'storage'), [order_point]),
+                    ]:
+                self.assertListEqual(
+                    OrderPoint.search(clause), result, msg=clause)
+
 
 def suite():
     suite = trytond.tests.test_tryton.suite()
